using System.Net;
using System.Text.Json;
using Amazon.Lambda.APIGatewayEvents;
using Amazon.Lambda.Core;
using Enyim.Caching;
using Enyim.Caching.Configuration;
using Enyim.Caching.Memcached;
using Microsoft.Extensions.Logging;

// Assembly attribute to enable the Lambda function's JSON input to be converted into a .NET class.
[assembly: LambdaSerializer(typeof(Amazon.Lambda.Serialization.SystemTextJson.DefaultLambdaJsonSerializer))]

namespace MemcachedLambdaFunctionURLs;

public class Function
{
    private MemcachedClient myCache;
    private readonly string functionId;
    private readonly string functionCreationTime; 
    private readonly Random random;
    
    public Function()
    {
        using var loggerFactory = LoggerFactory.Create(builder =>
        {
            builder.AddConsole();
        });
        ILogger logger = loggerFactory.CreateLogger<Function>();

        var config = new MemcachedClientConfiguration(loggerFactory, new MemcachedClientOptions());

        config.AddServer("xxxxxxxxxxxxxxxxxxx.xxxxxx.cfg.use1.cache.amazonaws.com", 11211);
        config.Protocol = MemcachedProtocol.Binary;
        myCache = new MemcachedClient(loggerFactory, config);
        
        functionId = Guid.NewGuid().ToString().Substring(0,8); 
        functionCreationTime = DateTime.Now.ToString("HH:mm:ss.fff");
        random = new Random();
    }

    public APIGatewayHttpApiV2ProxyResponse FunctionHandler(APIGatewayHttpApiV2ProxyRequest request, ILambdaContext context)
    {
        string itemId = request.QueryStringParameters["itemId"];
        Console.WriteLine($"Got itemId: {itemId}");

        ResponseBody responseBody = new ResponseBody() {
            FunctionId = functionId,
            FunctionCreationTime = functionCreationTime,
            ItemId = itemId
        };  
        
        if (myCache.TryGet(itemId, out string quantity))
        {
            responseBody.Quantity = quantity;
            responseBody.FromCache = true;
        }
        else
        {   
            quantity = random.Next(1000).ToString();
            responseBody.Quantity = quantity;
            responseBody.FromCache = false;
            myCache.SetAsync(itemId, quantity, 10);
        }
        
        var response = new APIGatewayHttpApiV2ProxyResponse
        {
            StatusCode = (int)HttpStatusCode.OK,
            Body = JsonSerializer.Serialize(responseBody),
            Headers = new Dictionary<string, string> { { "Content-Type", "application/json" } }
        };

        return response;
    }
}

public class ResponseBody
{
    public string FunctionId { get; set; }
    public string FunctionCreationTime { get; set; } 
    public string ItemId { get; set; }  
    public string Quantity { get; set; }
    public bool FromCache { get; set; }   
}